@@ -1446,7 +1446,8 @@ class Opt:
         """
         Generate a temperory name based on current machine time with a .tmp extension
         """
-        return 'T' + datetime.datetime.now().isoformat().replace(':', '')\
+        return 'T{}'.format(np.random.random().hex()) + \
+               datetime.datetime.now().isoformat().replace(':', '')\
             .replace('.', '').replace('-', '').replace('T', '') + '.tmp'
 
     @staticmethod
@@ -1484,52 +1485,6 @@ class Opt:
 
         return val
 
-    @staticmethod
-    def ogr_geom_type(input,
-                      format='num'):
-        """
-        Method to return OGR geometry type from input string
-        :param input: String to convert to OGR geometry type code
-        :param format: Which format should be output? (default: 'num'; options: 'num', 'name')
-        :return: OGR geometry type code
-        """
-
-        if format == 'num':
-            comp_str = input.lower()
-            comp_dict = {
-                'point': 1,
-                'line': 2,
-                'polygon': 3,
-                'multipoint': 4,
-                'multilinestring': 5,
-                'multipolygon': 6,
-                'geometry': 0,
-                'no geometry': 100
-            }
-            try:
-                return comp_dict[comp_str]
-            except (KeyError, NameError):
-                return None
-
-        elif format == 'name':
-            comp_dict = {
-                1: 'point',
-                2: 'line',
-                3: 'polygon',
-                4: 'multipoint',
-                5: 'multilinestring',
-                6: 'multipolygon',
-                0: 'geometry',
-                100: 'no geometry',
-            }
-            try:
-                return comp_dict[input].upper()
-            except (KeyError, NameError):
-                return None
-
-        else:
-            raise(ValueError('Invalid format'))
-
     @staticmethod
     def list_size(query_list):
         """
