@@ -1167,6 +1167,7 @@ class Vector(object):
                            out_epsg=4326,
                            attributes=None,
                            attribute_types=None,
+                           primary_key='fid',
                            verbose=False):
         """
         Make a vector object from a list of geometries in string (json, wkt, or wkb) format.
@@ -1176,6 +1177,7 @@ class Vector(object):
         :param spref_string: WKT representation of the spatial reference for the Vector object
         :param spref_string_type: Spatial reference string type (e.g. 'wkt', 'proj4', 'epsg'; default: 'wkt)
         :param out_epsg: EPSG SRID for the geometry object
+        :param primary_key: Primary key for features
         :param attributes: Dictionary or list of dictionaries of feature attributes.
                            The 'key' names in this list of dicts should match exactly with attribute_types
         :param attribute_types: Dictionary of feature attribute names with their OGR datatypes.
@@ -1205,15 +1207,19 @@ class Vector(object):
                      geom_type=vector_type.lower(),
                      in_memory=True,
                      verbose=verbose,
-                     primary_key='fid',
+                     primary_key=primary_key,
                      attr_def=attribute_types)
 
         if verbose:
             Opt.cprint('Creating geometries in memory...\n')
 
         for indx, geom_string in enumerate(geom_strings):
-            vector.add_feat(geom_func[geom_string_type](geom_string),
-                            attr=attributes[indx])
+
+            if attributes is None:
+                vector.add_feat(geom_func[geom_string_type](geom_string))
+            else:
+                vector.add_feat(geom_func[geom_string_type](geom_string),
+                                attr=attributes[indx])
 
         if verbose:
             Opt.cprint('Created {} geometries'.format(str(vector.nfeat)))
@@ -1370,7 +1376,7 @@ class Vector(object):
                                   y_max,
                                   0,
                                   -1.0*pixel_size[1]))
-       
+
         target_ds.SetProjection(target_ds_srs.ExportToWkt())
 
         if attribute is not None:
